/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: ircd.c,v 1.3 2000/06/16 13:15:01 wcampbel Exp $
*/

#include "stats.h"
extern const char version_date[], version_time[];

static void ShowMOTD(char *);

void parse(char *line)
{
	char *origin, *cmd, *coreLine;
	int cmdptr = 0;

	strip(line);

	if (!(*line))
		return;

	coreLine = strdup(line);

#ifdef DEBUG
	log("R: %s", line);
#endif
	origin = strtok(line, " ");
	if (*origin == ':')
		cmdptr = 1;
	cmd = strtok(NULL, " ");

	if (cmdptr) {
		origin++;
		if (!strcmp(cmd, "VERSION")) {
			sts(":%s 351 %s %s :%s -> %s %s", me.name, origin, version,
				me.name, version_date, version_time);
		} else if (!strcmp(cmd, "MOTD")) {
			ShowMOTD(origin);
		} else if (!strcmp(cmd, "PRIVMSG")) {
			cmd = strtok(NULL, " ");
			if (!strcasecmp(cmd, s_StatServ))
				statserv(coreLine);
			else if (!strcasecmp(cmd, s_Spam) && me.enable_spam)
				spam(coreLine);
		} else if (!strcmp(cmd, "SERVER")) {
			char *desc;
			cmd = strtok(NULL, " "); 
			desc = strtok(NULL, " ");
			desc = strtok(NULL, "\n");
			desc++;
			AddServer(cmd, 1, desc, origin);
		} else if (!strcmp(cmd, "SQUIT")) {
			Server *s;
			cmd = strtok(NULL, " ");
			if ((s = findserver(cmd))) s->stats->numsplits++;
			DelServer(cmd);
		} else if (!strcmp(cmd, "QUIT")) {
			DelUser(origin);
		} else if (!strcmp(cmd, "MODE")) {
			cmd = strtok(NULL, " ");
			if (!strchr(cmd, '#')) {
				cmd = strtok(NULL, "");
				cmd++;
				UserMode(origin, cmd);
			}
		} else if (!strcmp(cmd, "KILL")) {
			User *u = finduser(origin);
			Server *s;
			if (u) {
				u->server->stats->operkills++;
			} else {
				s = findserver(origin);
				if (s)
				s->stats->serverkills++;
			}
			cmd = strtok(NULL, " ");
			DelUser(cmd);
			if (!strcasecmp(cmd, s_StatServ))
				init_statserv();
			else if (!strcasecmp(cmd, s_Spam))
				init_spam();
		} else if (!strcmp(cmd, "PONG")) {
			Server *s;
			cmd = strtok(NULL, " ");
			s = findserver(cmd);
			if (s) {
				s->ping = time(NULL) - ping.last_sent;
				if (ping.ulag > 1)
					s->ping -= (float) ping.ulag;
				if (!strcmp(me.s->name, s->name))
					ping.ulag = me.s->ping;
				if (s->ping > s->highest_ping) {
					s->highest_ping = s->ping;
					s->t_highest_ping = time(NULL);
				} else if (s->ping < s->lowest_ping) {
					s->lowest_ping = s->ping;
					s->t_lowest_ping = time(NULL);
				}
			} else {
				log("Received PONG from unknown server: %s", cmd);
			}
		} else if (!strcmp(cmd, "AWAY")) {
			User *u = finduser(origin);
			if (u->is_away) {
				me.away--;
				u->is_away = 0;
			} else {
				me.away++;
				u->is_away = 1;
			}
		} else if (!strcmp(cmd, "NICK")) {
			User *u = finduser(origin);
			if (u) {
				cmd = strtok(NULL, " ");
				Change_User(u, cmd);
			}
		}
	} else {
		if (!strcmp(origin, "PING")) {
			sts("PONG %s", cmd);
		} else if (!strcmp(origin, "PASS")) {
			init_statserv();
			init_spam();
			init_join();
		} else if (!strcmp(origin, "SERVER")) {
			char *desc;
			Server *s;
			desc = strtok(NULL, " ");
			desc = strtok(NULL, "\n");
			desc++;
			AddServer(cmd, 1, desc, me.name);
			s = findserver(cmd);
			me.s = s;
		} else if (!strcmp(origin, "SQUIT")) {
			Server *s = findserver(cmd);
			if (s) s->stats->numsplits++;
			DelServer(cmd);
		} else if (!strcmp(origin, "NICK")) {
		  if (!strcmp(me.ircd_type, "hybrid") ||
		      !strcmp(me.ircd_type, "hybrid6")) {
			char *user, *host, *server, *modes; 
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			modes = strtok(NULL, " "); 
			user = strtok(NULL, " ");
			host = strtok(NULL, " ");
			server = strtok(NULL, " ");
			AddUser(cmd, user, host, server);
			UserMode(cmd, modes); 
		  } else if (!strcmp(me.ircd_type, "dal")) {
			char *user, *host, *server;
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			user = strtok(NULL, " ");
			host = strtok(NULL, " ");
			server = strtok(NULL, " ");
			AddUser(cmd, user, host, server);
		  }
                } else if (!strcmp(origin, "SVSNICK")) {
                        char *nnick;
                        User *u;
       
                        nnick = strtok(NULL, " ");
                        u = finduser(nnick);
                        nnick = strtok(NULL, " ");
                        Change_User(u, nnick);
                }

	}
	free(coreLine);
}

void privmsg(char *to, const char *from, char *fmt, ...)
{
	va_list ap;
	char buf[512], buf2[512];

	va_start(ap, fmt);
	vsnprintf(buf2, sizeof(buf2), fmt, ap);
	if (me.want_privmsg)
		sprintf(buf, ":%s PRIVMSG %s :%s", from, to, buf2);
	else
		sprintf(buf, ":%s NOTICE %s :%s", from, to, buf2);
	sts("%s", buf);
	va_end(ap);
}

void privmsg_list(char *to, char *from, const char **text)
{
	while (*text) {
		if (**text)
			privmsg(to, from, "%s", *text);
		else
			privmsg(to, from, " ");
		text++;
	}	
}

void globops(char *from, char *fmt, ...)
{
	va_list ap;
	char buf[512], buf2[512];

	va_start(ap, fmt);
	vsnprintf(buf2, sizeof(buf2), fmt, ap);
	if (!strcmp(me.ircd_type, "hybrid") ||
	    !strcmp(me.ircd_type, "hybrid6")) {
		sprintf(buf, ":%s WALLOPS :%s", from, buf2);
	} else if (!strcmp(me.ircd_type, "dal")) {
		sprintf(buf, ":%s GLOBOPS :%s", from, buf2);
	}
	sts("%s", buf);
	va_end(ap);
}

int flood(User *u)
{
	time_t current = time(NULL);

	if (current - u->t_flood > 10) {
		u->t_flood = time(NULL);
		u->flood = 0;
		return 0;
	}
	if (u->flood >= 5) {
		sts(":%s KILL %s :%s!%s (Flooding Services.)", s_StatServ,
			u->nick, StatServ.host, s_StatServ);
		log("FLOODING: %s!%s@%s", u->nick, u->username, u->hostname);
		return 1;
	} else {
		u->flood++;
	}
	return 0;
}

static void ShowMOTD(char *nick)
{
	static unsigned char copyright[38] = {  /* Can you tell I was bored? */
	0x47, 0x65, 0x6f, 0x53, 0x74, 0x61, 0x74, 0x73, 0x20, 0x61, 0x72, 0x65,
	0x20, 0x28, 0x63, 0x29, 0x20, 0x31, 0x39, 0x39, 0x39, 0x20, 0x4a, 0x6f,
	0x6e, 0x61, 0x74, 0x68, 0x61, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
	0x65, 0x0
	};

	/* Removal or modification of this function is against the law.
	** Be nice, and don't remove it or modify it.
	** -Jonathan 'netgod' George (net@lite.net) */

	sts(":%s 375 %s :- %s Message of the Day", me.name, nick, me.name);
	sts(":%s 372 %s :- ", me.name, nick);
	sts(":%s 372 %s :- %s", me.name, nick, copyright);
	sts(":%s 372 %s :- Grab your copy at ftp.lite.net", me.name, nick);
	sts(":%s 376 %s :End of /MOTD command.", me.name, nick);
}

void SendQuit(User *u)
{
	sts(":%s QUIT :Shutdown Request by %s", s_StatServ, u->nick);
	if (me.enable_spam)
		sts(":%s QUIT :Shutdown Request by %s", s_Spam, u->nick);
}
